(* $Id: map.ml,v 1.1 1996/12/10 09:02:51 garrigue Exp $ *)

(* Single-binding association tables over ordered types *)

#open "list";;
#open "baltree";;

type ('a, 'b) binding =
  { key: 'a; data: 'b };;

type ('a, 'b) t =
  { tree: ('a, 'b) binding baltree__t;
    order: 'a -> ('a, 'b) binding -> int };;

let empty ord =
  { tree = Empty; order = fun x y -> ord x y.key };;

let add x y m =
  { tree = baltree__modify (m.order x)
             (fun _ -> Something {key=x; data=y}) m.tree;
    order = m.order };;

let find x m =
  (baltree__find (m.order x) m.tree).data;;

let remove x m =
  { tree = baltree__modify (m.order x) (fun _ -> Nothing) m.tree;
    order = m.order };;

let iter f m =
  let rec iter = function
    Empty -> ()
  | Node(l, b, r, _) ->
      iter l;
      f b.key b.data;
      iter r
  in iter m.tree;;
